package org.steveshipway.blueprints;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Random;
import java.util.Set;

import org.apache.logging.log4j.Logger;
import org.steveshipway.blueprints.lib.Constants;

import net.minecraft.block.Block;
import net.minecraft.init.Blocks;
import net.minecraft.world.World;
import net.minecraft.world.biome.BiomeGenBase;
import net.minecraft.world.chunk.IChunkProvider;
import net.minecraft.world.gen.feature.WorldGenMinable;

// Minecraft 1.7
import org.steveshipway.blueprints.util17.*;
import cpw.mods.fml.common.IWorldGenerator;
// Minecraft 1.8
//import net.minecraft.util.BlockPos;
//import org.steveshipway.blueprints.util18.*;
//import net.minecraftforge.fml.common.IWorldGenerator;
	 
public class AutoGenerator implements IWorldGenerator
{
	static Logger logger = null;
	static Portfolio portfolio = null;
	public AutoGenerator(Portfolio portfolio, Logger l) {
		super();
		this.logger = l;
		this.portfolio = portfolio;
	}
	public AutoGenerator() {
		super();
	}
	@Override
	public void generate(Random random, int chunkX, int chunkZ, World world, IChunkProvider chunkGenerator, IChunkProvider chunkProvider)
	{
		Blueprint bp = null;
		String bpName = null;
		BlockPos location = new BlockPos(chunkX*16, 0, chunkZ*16);
		int r;
		Block blk;
		//logger.info("Calling generate");
		if( world.isRemote ) { return; } // should never happen, but just in case
		//logger.info("Not remote");
		
		String biome = Util.getBiome(world,location);
		
        // now, loop through all known structures, and search for one with Auto enabled, that is randomly selected, and matches this biome.
        Iterator i = this.portfolio.auto_iterator();
        while( i.hasNext() ) {
        	bpName = (String) i.next(); 
        	bp = this.portfolio.blueprint(bpName);
        	//logger.info("Testing plan "+bpName+" for chance "+bp.auto_chance+"/"+Constants.CHANCE_BASE);
        	if( bp.auto_chance == 0 ) { continue; }              // not auto-enabled
        	//logger.info("Testing auto-plan "+bpName+" for chance "+bp.auto_chance+"/"+Constants.CHANCE_BASE);
        	if( random.nextInt(Constants.CHANCE_BASE) > (bp.auto_chance*Constants.CHANCE_BASE) ) { continue; } // not randomly selected
        	//logger.info("Checking biome list for "+bpName+ " against "+b.biomeName);
        	if( !bp.biomes.contains(biome) && !bp.biomes.contains("Any")) { continue; } // not matching biome name
        	// can we probe the chunk for torches and doors (the presence of which would indicate existing structures)?
        	// we're doing this one!
        	r = random.nextInt(4);
        	int x, y, z;
        	x = random.nextInt(16) + ( chunkX * 16 );
        	z = random.nextInt(16) + ( chunkZ * 16 );
//        	y = world.getHeightValue(location.getX(), location.getZ()); // how to do this in 1.8?
        	y = 128; // just sink down, then.
        	location = new BlockPos(x,y,z);
    		// float up if underground
    		//while( location.posY < 128 && world.getBlock(location) != Blocks.air 
    		//		&& world.getBlock(location) != Blocks.water ) {
    		//	location.posY += 1; 
    		//}
        	// sink to ground if in the air

        	Set<Block> floatthrough = new HashSet<Block>();    		
        	// we go down through air, plants, and snow
   			floatthrough.add(Blocks.air); 
    		floatthrough.add(Blocks.log); // treat trees as air
    		floatthrough.add(Blocks.log2);
    		floatthrough.add(Blocks.leaves);
    		floatthrough.add(Blocks.leaves2);
    		floatthrough.add(Blocks.yellow_flower); // treat flowers and plants as air
    		floatthrough.add(Blocks.red_flower);
    		floatthrough.add(Blocks.double_plant);
    		floatthrough.add(Blocks.wheat);
    		floatthrough.add(Blocks.pumpkin);
    		floatthrough.add(Blocks.melon_block);
    		floatthrough.add(Blocks.pumpkin_stem);
    		floatthrough.add(Blocks.melon_stem);
    		floatthrough.add(Blocks.waterlily);
    		floatthrough.add(Blocks.reeds);
    		floatthrough.add(Blocks.cactus);
    		floatthrough.add(Blocks.potatoes);
    		floatthrough.add(Blocks.tallgrass);
   			floatthrough.add(Blocks.fire);
   			floatthrough.add(Blocks.snow);  // down through snow
   			floatthrough.add(Blocks.snow_layer);
    		// float down to ground: check what is under us
    		blk = Util.getBlock(world,location.down());
    		while( location.getY() > 0 && floatthrough.contains(blk) ) {
    			location = location.down();
    			blk = Util.getBlock(world,location.down());
    		}
    		
    		if( location.getY()<1 ) { break; } // invalid location, are you using skylands?

        	if( this.logger != null && ! portfolio.quiet ) {
        		this.logger.info("Autogenerating a "+bpName+ " at "+location.getX()+","+location.getY()+","+location.getZ()+" rot="+r);
        	}
        	bp.build(world, location, r);
        	break; // only one structure per chunk
        }
	}
	
}
