package org.steveshipway.blueprints.util17;

// All Minecraft 1.7 compatibility functions should be in here!

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.Random;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.logging.log4j.Logger;

import net.minecraft.block.Block;
import net.minecraft.entity.Entity;
import net.minecraft.init.Blocks;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.Item;
import net.minecraft.nbt.CompressedStreamTools;
import net.minecraft.nbt.JsonToNBT;
import net.minecraft.nbt.NBTBase;
import net.minecraft.nbt.NBTException;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagDouble;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.WeightedRandomChestContent;
import net.minecraft.world.World;
import net.minecraftforge.common.util.Constants;
import net.minecraftforge.common.ChestGenHooks;


// Minecraft 1.7
import org.steveshipway.blueprints.util17.*;

import cpw.mods.fml.common.registry.GameRegistry;

public class Util {   
	public static Pattern nbtfile = Pattern.compile("\\s*(\\S+\\.nbt)\\s*");
	
	public Util() {
	}
	
	// rotate state by r x 90degrees
    public static IBlockState rotateState( IBlockState blockstate, int r ) {
    	Block b = blockstate.getBlock();
    	IBlockState newstate;
    	int newDv = blockstate.getDv();
    	
        final Map<Integer, Integer> rotate = new HashMap<Integer, Integer>();

    	while( r > 3 ) { r -= 4; }
    	while( r < 0 ) { r += 4; }
    	if( r == 0 ) { return blockstate; } // null operation

    	// only certain blocks have orientation, and some have only 2 relevant orientations.
    	if( b == Blocks.log || b == Blocks.log2 || b == Blocks.hay_block ) {
    		// only need quarter-turns, and only for horizontal alignments
    		// swap the top two bits
    		if( r == 1 || r == 3) {
    			if( newDv >= 4 && newDv <= 7 ) {
    				newDv += 4;
    			} else if( newDv >= 8 && newDv <= 11) {
    				newDv -= 4;
    			}
    		}
    	} else if( b == Blocks.torch || b == Blocks.redstone_torch 
    			|| b == Blocks.stone_button || b == Blocks.wooden_button
    	) {
    		// Standard 1-3-2-4 rotation, do not rotate 0 and 5, preserve top bit
            rotate.put(1, 3); rotate.put(2, 4);
            rotate.put(3, 2); rotate.put(4, 1);
            rotate.put(5, 5); rotate.put(0, 0);
    		int i = r;
    		while(i>0) { i-=1; newDv = rotate.get(newDv & 7) | (newDv & 8); }
    	} else if( b == Blocks.lever ) {
    		// Standard 1-3-2-4 rotation, 5/6 and 7/0 swap, preserve top bit
            rotate.put(1, 3); rotate.put(2, 4);
            rotate.put(3, 2); rotate.put(4, 1);
            rotate.put(5, 6); rotate.put(6, 5);
            rotate.put(7, 0); rotate.put(0, 7);
    		int i = r;
    		while(i>0) { i-=1; newDv = rotate.get(newDv & 7) | (newDv & 8); }
    	} else if( b == Blocks.bed || b == Blocks.lit_pumpkin || b == Blocks.pumpkin 
    			|| b == Blocks.powered_repeater || b == Blocks.unpowered_repeater
    			|| b == Blocks.unpowered_comparator || b == Blocks.powered_comparator 
    			|| b == Blocks.trapdoor || b == Blocks.end_portal_frame
    			|| b == Blocks.fence_gate 
    			|| b == Blocks.cocoa || b == Blocks.tripwire_hook
    			|| b == Blocks.anvil ) {
    		// 0-1-2-3 rotation preserving high bits
    		int i = (( newDv & 3 ) + r ) & 3;
    		newDv = (newDv & 12) | i;
    	} else if( b == Blocks.piston || b == Blocks.piston_head || b == Blocks.piston_extension  || b == Blocks.sticky_piston
    			|| b == Blocks.cobblestone_wall 
    			|| b == Blocks.activator_rail || b == Blocks.detector_rail || b == Blocks.golden_rail
    			|| b == Blocks.ladder || b == Blocks.chest || b == Blocks.ender_chest || b == Blocks.trapped_chest  
    			|| b == Blocks.furnace || b == Blocks.lit_furnace
    			|| b == Blocks.wall_sign || b == Blocks.dropper || b == Blocks.hopper || b == Blocks.dispenser
    			|| b == Blocks.skull
    			) {
    		// 2-5-3-4 rotation preserving top bit
            rotate.put(2, 5); rotate.put(5, 3);
            rotate.put(3, 4); rotate.put(4, 2);
            rotate.put(0, 0); rotate.put(1, 1);
    		int i = r;
    		//logger.info("Block: " + Block.blockRegistry.getNameForObject(b) + "  Rot: " + r + "  DV: " + newDv);
    		while(i>0) { 
    			i-=1; newDv = rotate.get(newDv & 7) | (newDv & 8); 
    			//logger.info("Rotate -- DV: " + newDv);
    		}
    	} else if( b == Blocks.stone_stairs || b == Blocks.stone_brick_stairs || b == Blocks.acacia_stairs || b == Blocks.birch_stairs
    			|| b == Blocks.brick_stairs || b == Blocks.nether_brick_stairs || b == Blocks.dark_oak_stairs || b == Blocks.jungle_stairs
    			|| b == Blocks.oak_stairs || b == Blocks.quartz_stairs || b == Blocks.sandstone_stairs || b == Blocks.spruce_stairs ) {
    	 	// 3-0-2-1 stairs rotation preserving upper bits
            rotate.put(3, 0); rotate.put(0, 2);
            rotate.put(2, 1); rotate.put(1, 3);
    		int i = r;
    		while(i>0) { i-=1; newDv = rotate.get(newDv & 3) | (newDv & 12); }
    	} else if( b == Blocks.wooden_door || b == Blocks.iron_door  ) {
    		// 0-1-2-3 on bottom 2 bits if top bit unset
    		if( newDv <= 7 ) { 
    			int i = (( newDv & 3 ) + r ) & 3;
    			newDv = (newDv & 12) | i;
    		}
    	} else if( b == Blocks.rail ) {
    		// 2-5-3-4 with 0-1 and 6-7-8-9
            rotate.put(2, 5); rotate.put(5, 3);
            rotate.put(3, 4); rotate.put(4, 2);
            rotate.put(0, 1); rotate.put(1, 0);
            rotate.put(6, 7); rotate.put(7, 8);
            rotate.put(8, 9); rotate.put(9, 6);
    		int i = r;
    		while(i>0) { i-=1; newDv = rotate.get(newDv); }    		
        }  else if( b == Blocks.standing_sign ) {
        	newDv = ( newDv - 4 );  if( newDv < 0) { newDv += 15; }
        } else if( b == Blocks.brown_mushroom_block || b == Blocks.red_mushroom_block ) {
            rotate.put(1, 3); rotate.put(3, 9);
            rotate.put(9, 7); rotate.put(7, 1);
            rotate.put(2, 6); rotate.put(6, 8);
            rotate.put(8, 4); rotate.put(4, 2);
            rotate.put(0, 0); rotate.put(5, 5);
            rotate.put(10, 10); 
            rotate.put(14, 14); rotate.put(15, 15);
    		int i = r;
    		while(i>0) { i-=1; newDv = rotate.get(newDv); }
        } else if( b == Blocks.vine ) {
    		int i = r;
    		while(i>0) { 
    			i-=1; 
            	newDv = newDv * 2;
            	if( newDv > 15 ) { newDv -= 15; } // rotate top bit to lowest 
    		}
        } else if( b == Blocks.quartz_block ) {
            rotate.put(4, 3); rotate.put(3, 4);
            rotate.put(0, 0); rotate.put(1, 1);
            rotate.put(2, 2); 
    		int i = r;
    		while(i>0) { i-=1; newDv = rotate.get(newDv); }    		
        }
    	
    	newstate = new IBlockState(b,newDv);
		return newstate;
    }
    
    // Utility function.  Convert "{nbt}" or "file.nbt" into a parsed NBT tag
    public static NBTTagCompound parseNBT(String s, String bppath, Logger logger) {
    	NBTTagCompound nbtTag = null;
    	Matcher m = nbtfile.matcher(s);
    	File nbtfile;
    	if( m.find() ) {
    		// we have an NBT file
    		if( s.startsWith(File.separator) || bppath == null ) {
    			nbtfile = new File(m.group(1));
    		} else {
    			nbtfile = new File( bppath+File.separator+"nbt"+File.separator+m.group(1) );
    		}
    		if( ! nbtfile.exists() ) {
				logger.error("Unable to read file "+m.group(1)+" : File does not exist in "+bppath+File.separator+"nbt"+File.separator);
    			return null;
    		} 
    		try {
				nbtTag = CompressedStreamTools.readCompressed(new FileInputStream(nbtfile));
			} catch (IOException e) {
				logger.error("Unable to read file "+m.group(1)+" : "+e.toString());
				return null;
			}
    	} else {
			try {
				// seek out the only function in JsonToNBT that takes a string and returns an NBTTag
				// Minecraft 1.7
		    	nbtTag = (NBTTagCompound) JsonToNBT.func_150315_a(s); // forge 1.7.10
		    	// Minecraft 1.8
				//nbtTag = (NBTTagCompound) JsonToNBT.func_180713_a(s); // forge 1.8
			} catch (NBTException e) {
				nbtTag = null;
				if( logger != null ) {
					logger.error(s);
					logger.error("Unable to parse NBT: "+e.getMessage());
				}
				return null;
			} 
    	}
    	return nbtTag;
    }
    
    // Abstract out getBlock() for portability
    public static Block getBlock(World world, BlockPos location) {
    	Block blk;
    	// Minecraft 1.8
    	//blk =  world.getBlockState(location).getBlock();
    	// Minecraft 1.7
    	blk =  world.getBlock(location.getX(),location.getY(),location.getZ());
    	return blk;
    }
    
    public static IBlockState getState(Block b,int dv) {
    	IBlockState bs;
    	
    	// Minecraft 1.7
    	bs = new IBlockState(b, dv);
    	// Minecraft 1.8
    	//bs = b.getBlockState().getBaseState().getBlock().getStateFromMeta(dv);
    	
    	return bs;
    }
    public static IBlockState getBlockState(World world, BlockPos blockPos) {
    	IBlockState bs;
    	// Minecraft 1.7
		Block curBlock = world.getBlock(blockPos.getX(), blockPos.getY(), blockPos.getZ());
		int curDv = world.getBlockMetadata(blockPos.getX(), blockPos.getY(), blockPos.getZ());
		bs = new IBlockState(curBlock,curDv);
    	// Minecraft 1.8
		//bs = world.getBlockState(blockPos);
		 
		return bs;
    }
    public static boolean setBlockState(World world, BlockPos blockPos, IBlockState bs) {
    	boolean rv;
    	
		//CheshireCat.portfolio.getlogger().info("Setting block "+bs.getBlock().toString()+" DV="+bs.getDv());
    	
    	// Minecraft 1.7
    	rv = world.setBlock(blockPos.getX(),blockPos.getY(),blockPos.getZ(),bs.getBlock(),bs.getDv(),2);
    	// Minecraft 1.8
    	//rv = world.setBlockState(blockPos,bs,2);
    	
    	return rv;
    }
    public static int getDvFromState(IBlockState bs) {
    	int dv;
    	
    	// Minecraft 1.7
    	dv = bs.getDv();
    	// Minecraft 1.8
    	//dv = bs.getBlock().getMetaFromState(bs);
    	
    	return dv;
    }
    public static void setTileEntity(World world,BlockPos blockPos,TileEntity te) {
		// Minecraft 1.8
		//world.removeTileEntity(blockPos); // should we instead get and merge?
		//world.setTileEntity(blockPos,te);
		// Minecraft 1.7
		world.removeTileEntity(blockPos.getX(),blockPos.getY(),blockPos.getZ()); // should we instead get and merge?
		world.setTileEntity(blockPos.getX(),blockPos.getY(),blockPos.getZ(),te);

    }
    public static void setTileEntity(World world,BlockPos blockPos,NBTTagCompound nbtTag) {
    	TileEntity te;
    	NBTTagCompound oldTag;
    	NBTTagCompound newTag;

    	// temporary
		//CheshireCat.portfolio.getlogger().info("Creating new tile entity from tag");

		te = getTileEntity(world,blockPos);

		oldTag = new NBTTagCompound();
		if( te != null ) {
			te.writeToNBT(oldTag); // note this does not remove existing tags
			oldTag.removeTag("x"); // remove the tags we don't need
			oldTag.removeTag("y");
			oldTag.removeTag("z");
			newTag = mergeNbt(oldTag,nbtTag);
			// Minecraft 1.8
			//world.removeTileEntity(blockPos); // should we instead get and merge?
			// Minecraft 1.7
			world.removeTileEntity(blockPos.getX(),blockPos.getY(),blockPos.getZ()); 
		} else {
			newTag = (NBTTagCompound) nbtTag.copy();
			// temporary
			//CheshireCat.portfolio.getlogger().info("No existing tile entity found");
		}
		// temporary
		//CheshireCat.portfolio.getlogger().info("Setting id="+newTag.getString("id"));
		//CheshireCat.portfolio.getlogger().info("Setting Items="+newTag.getTag("Items").toString());
		te = TileEntity.createAndLoadEntity(newTag);
    	
		// Minecraft 1.7
		world.setTileEntity(blockPos.getX(),blockPos.getY(),blockPos.getZ(),te);
		// Minecraft 1.8
		//world.setTileEntity(blockPos,te);

    }
    public static TileEntity getTileEntity(World world, BlockPos blockPos) {
    	// Minecraft 1.7
    	return world.getTileEntity(blockPos.getX(),blockPos.getY(),blockPos.getZ());
    	// Minecraft 1.8
    	//return world.getTileEntity(blockPos);
    }
    public static Item getItemByNameOrId(int id) {
    	// Minecraft 1.7
    	return Item.getItemById(id);
    	// Minecraft 1.8
    	//return Item.getByNameOrId(id);    	
    }
    public static Item getItemByNameOrId(String s) {
    	// This works for 1.7, but doesn't allow bare object names
        //return (Item) Item.itemRegistry.getObject(s);
    	// Minecraft 1.7
        String modId = "minecraft"; // default
        String name;
        if (s.indexOf(":") != -1) {
            modId = s.substring(0, s.indexOf(":"));
        }
        name = s.substring(s.indexOf(":") + 1);
        return GameRegistry.findItem(modId, name); // Will return null if no item is found.
    	// Minecraft 1.8
    	//return Item.getByNameOrId(s);
    }
    public static EnumFacing enumFromInt(int facing) {
    	EnumFacing side = net.minecraft.util.EnumFacing.UP;
    	switch ( facing ) {
	    	case 0: side = net.minecraft.util.EnumFacing.DOWN; break;  //
	    	case 1: side = net.minecraft.util.EnumFacing.UP; break;    //
	    	case 2: side = net.minecraft.util.EnumFacing.SOUTH; break; //
	    	case 3: side = net.minecraft.util.EnumFacing.NORTH; break; //
	    	case 4: side = net.minecraft.util.EnumFacing.EAST; break;  //
	    	case 5: side = net.minecraft.util.EnumFacing.WEST; break;  //
    	}
    	return side;
    }
    public static NBTTagCompound mergeNbt(NBTTagCompound oldTag,NBTTagCompound replaceTag) {
    	NBTTagCompound newTag;
    	
    	// temporary
		//CheshireCat.portfolio.getlogger().info("Merging NBT");
		
		if( oldTag == null ) {
			return ((NBTTagCompound) replaceTag.copy());
		}
		newTag = (NBTTagCompound) oldTag.copy();
		if( replaceTag == null ) {
			return newTag;
		}
		
    	// step through all tags in replaceTag and put them into newTag
    	for( Object k : replaceTag.func_150296_c() ) {
    		if( k instanceof String ) {
    			newTag.removeTag((String)k);
    			newTag.setTag((String)k, replaceTag.getTag((String)k));
    		} else {
    			// temporary
    			//CheshireCat.portfolio.getlogger().info("Function returned type "+k.toString());
    		}
    	}
    	return newTag;
    }
    // fix the tag if it is in MC 1.8 format
    public static NBTTagCompound fixChestNbt(Logger logger, NBTTagCompound nbt) {
    	NBTTagList items;
    	
    	nbt.removeTag("id");
    	nbt.setString("id","Chest");
    	items = nbt.getTagList("Items",Constants.NBT.TAG_COMPOUND);
    	if( items != null ) {
	    	for (int i = 0; i < items.tagCount(); ++i) {
	    		NBTTagCompound item = (NBTTagCompound) items.getCompoundTagAt(i);
	    		if( !item.hasKey("id",Constants.NBT.TAG_SHORT) ) {
	    			String s = item.getString("id");
	    			if( s == null ) { s = "minecraft:stick"; }
	    			Item thing = getItemByNameOrId(s);
	    			short itemno = 280;
	    			if( thing != null ) {
	    				itemno = (short) Item.getIdFromItem(thing);
	    			}
	    			logger.warn("Correcting NBT for chest item from "+s+" to "+itemno);
	    			item.setShort("id", itemno);
	    		}
	    	}
    	}
    	return nbt;
    }
    public static NBTTagCompound fixSpawnerNbt(Logger logger, NBTTagCompound nbt) {
    	NBTTagList potentials;
    	
    	if( !nbt.hasKey("id",Constants.NBT.TAG_STRING) || !(nbt.getString("id") == "MobSpawner") ) {
    		logger.warn("Error in spawner NBT: Setting id to MobSpawner");
    		nbt.setString("id", "MobSpawner");
    	}
    	    	
    	// this can stop crashes under 1.7
    	potentials = nbt.getTagList("SpawnPotentials",Constants.NBT.TAG_COMPOUND);
    	if( potentials != null ) {
	    	for (int i = 0; i < potentials.tagCount(); ++i) {
	    		NBTTagCompound pot = (NBTTagCompound) potentials.getCompoundTagAt(i);
	    		if( !pot.hasKey("Type",Constants.NBT.TAG_STRING) || (pot.getString("Type").length()<1) ) {
	    			logger.warn("No Spawner potential Type set:  Setting Type to Chicken");
	    			pot.setString("Type", "Chicken");
	    		}
	    	}
    	}

		if( ((nbt.getString("EntityId")==null)||(nbt.getString("EntityId").length()<1)) && (potentials.tagCount() > 0) ) {
			try {
				NBTBase firstPotential =  Util.getIndex(potentials, 0);				
				nbt.setString("EntityId", ((NBTTagCompound)firstPotential).getString("Type") );
				nbt.setTag("SpawnData", ((NBTTagCompound)firstPotential).getTag("Properties").copy() );
			} catch( Exception e ) {
				logger.warn("Possible problem in duplicating primary spawn potential to base definition");
				logger.warn("Error: "+e.toString());
			}
		}

    	// this stops crashes under 1.7 where entityid is unset or null
    	// it doesnt validate that the entityId is real, though
    	if( !nbt.hasKey("EntityId",Constants.NBT.TAG_STRING) || (nbt.getString("EntityId").length() < 1) ) {
    		logger.warn("Error in spawner NBT:  No EntityId set: Setting entity to Chicken");
    		nbt.setString("EntityId", "Chicken");
    	}
    	
    	//logger.info("Spawner NBT tag: "+nbt.toString());
    	return nbt;
    }
    
    public static Entity getEntityByID(World world, int entityId)
    {
   	 for(Object o: world.getLoadedEntityList())
        {
            if(((Entity)o).getEntityId() == entityId )
            {
                return ((Entity)o);
            }
        }
        return null;
    }
    
    public static NBTTagCompound getIndex(NBTTagList list,int index) {
		// Minecraft 1.8
		//return list.get(index);
		// Minecraft 1.7
		return list.getCompoundTagAt(index);

    }
    public static String getBiome(World world, BlockPos location ) {
		// Minecraft 1.8
        //return world.getBiomeGenForCoords(location).biomeName;
        // Minecraft 1.7
        return world.getBiomeGenForCoords(location.getX(),location.getZ()).biomeName;    	
    }
    // convert a metablock type to a block
    public static IBlockState metaBlock(World world, BlockPos pos, String name, int dv) {
    	Random r = new Random();
    	Block b = Blocks.air;
    	IBlockState bs;
        String biome = Util.getBiome(world,pos);
    	if( name.contentEquals( "STONE") ) {
    		// change according to biome
    		// change to sandstone if in desert
    		if( biome.contains("Desert") ) {
        		b = Blocks.sandstone;
    		} else if(biome.contains("Hell") ) {
    			b = Blocks.nether_brick;
    		} else if(biome.contains("Mesa") ) {
    			b = Blocks.sandstone;
    		} else {
    			switch( r.nextInt(10) ) {
    			case 0:
    				b = Blocks.monster_egg;
    				dv = 1; // cobblestone; 3=mossy
    				break;
    			case 1:
    			case 2:
    				b = Blocks.stonebrick;
    				break;
    			default:
    				b = Blocks.cobblestone;
    			}
    		}
    	} else if( name.contentEquals( "STONE_SLAB") ) {
    		// change according to biome
    		// change to sandstone if in desert
    		if( biome.contains("Desert") ) {
        		b = Blocks.stone_slab;
    			if(dv>7) { dv = 9; } else { dv = 1; }
    		} else if(biome.contains("Hell") ) {
    			b = Blocks.stone_slab;
    			if(dv>7) { dv = 14; } else { dv = 6; }
    		} else {
   				b = Blocks.stone_slab;
    			if(dv>7) { dv = 11; } else { dv = 3; }
  			}
    	} else if( name.contentEquals( "STONE_STEPS") ||  name.contentEquals( "STONE_STAIRS" )) {
    		// change according to biome
    		// preserve original DV
    		if( biome.contains("Desert") ) {
        		b = Blocks.sandstone_stairs;
    		} else if(biome.contains("Hell") ) {
    			b = Blocks.nether_brick_stairs;
    		} else if(biome.contains("Mesa") ) {
    			b = Blocks.sandstone_stairs;
    		} else {
        		b = Blocks.stone_stairs;
    		}
    	} else if( name.contentEquals( "OLD_STONE" )) {
    		// change according to biome
    		if( biome.contains("Desert") ) {
           		b = Blocks.sandstone;
    		} else if(biome.contains("Hell") ) {
    			b = Blocks.nether_brick;
    		} else {
        		// can be mossy or have silverfish
    			switch( r.nextInt(10) ) {
    			case 0:
    				b = Blocks.monster_egg;
    				dv = 1; // cobblestone 
    				break;
    			case 1:
    				b = Blocks.monster_egg;
    				dv = 3; // mossy brick
    				break;
    			case 2:
    			case 3:
    				b = Blocks.mossy_cobblestone;
    				break;
    			case 4:
    				dv = 0;
    				b = Blocks.stonebrick;
    				break;
    			case 5:
    				dv = 1; // mossy brick
    				b = Blocks.stonebrick;
    				break;
    			case 6:
    				dv = 2; // cracked brick
    				b = Blocks.stonebrick;
    				break;
            	default:
            		b = Blocks.cobblestone;
            		break;
			}

    		}
    	} else if( name.contentEquals( "WOOD" ) || name.contentEquals( "PLANKS" ) ) {
    		// change DV according to biome.  order of tests is important
    		b = Blocks.planks;
			if( biome.matches(".*(Ice|Taiga).*") ) { // spruce
				dv = 1;
			} else if( biome.matches("(Roofed|Hell).*") ) { // dark oak
				dv = 5;
			} else if( biome.matches("Jungle.*") ) { // jungle
				dv = 3;
			} else if( biome.matches("Savanna.*") ) { // acacia
				dv = 4;
			} else if( biome.matches(".*(Plains|Forest|Desert|Mesa|Plateau).*") ) { // birch
				dv = 2;
			} else { // oak
				dv = 0;
			}
    	} else if( name.contentEquals( "WOOD_STEPS" ) ||  name.contentEquals( "WOOD_STAIRS" )) {
    		// change according to biome
			if( biome.matches(".*(Ice|Taiga).*") ) { // spruce
				b = Blocks.spruce_stairs;
			} else if( biome.matches("(Roofed|Hell).*") ) { // dark oak
				b = Blocks.dark_oak_stairs;
			} else if( biome.matches("Jungle.*") ) { // jungle
				b = Blocks.jungle_stairs;
			} else if( biome.matches("Savanna.*") ) { // acacia
				b = Blocks.acacia_stairs;
			} else if( biome.matches(".*(Plains|Forest|Desert|Mesa|Plateau).*") ) { // birch
				b = Blocks.birch_stairs;
			} else { // oak
				b = Blocks.oak_stairs;
			}
		} else if( name.contentEquals( "WOOD_SLAB" )) {
    		b = Blocks.wooden_slab;
    		if( dv > 7 ) { dv = 8; } else { dv = 0; } // upper or lower
			if( biome.matches(".*(Ice|Taiga).*") ) { // spruce
				dv += 1;
			} else if( biome.matches("(Roofed|Hell).*") ) { // dark oak
				dv += 5;
			} else if( biome.matches("Jungle.*") ) { // jungle
				dv += 3;
			} else if( biome.matches("Savanna.*") ) { // acacia
				dv += 4;
			} else if( biome.matches(".*(Plains|Forest|Desert|Mesa|Plateau).*") ) { // birch
				dv += 2;
			}  // oak
    	} else if( name.contentEquals("DOOR") ) { // only one option in MC1.7
			b = Blocks.wooden_door;
    	} else if( name.contentEquals("FENCE") ) { // only one option in MC1.7
			b = Blocks.fence;
    	} else if( name.contentEquals("GATE") ) { // only one option in MC1.7
    		b = Blocks.fence_gate;
    	} else if( name.contentEquals("GLASS") ) {  // random coloured glass pane
    		b = Blocks.stained_glass;
    		dv = r.nextInt(16);
    	} else if( name.contentEquals("GLASS_PANE") ) {  // random coloured glass pane
    		b = Blocks.stained_glass_pane;
    		dv = r.nextInt(16);
    	} else {
    		b = Block.getBlockFromName(name);
    		if( b == null ) {
    			b = Blocks.air;
    		}
    	}
    	bs = Util.getState(b,dv);
    	return bs;
    }
    public static AxisAlignedBB getBB(BlockPos from, BlockPos to) {
    	AxisAlignedBB aabb;
    	
    	// Minecraft 1.7
    	double fx,fy,fz,tx,ty,tz;
    	if( from.getX() > to.getX() ) { fx=to.getX(); tx=from.getX(); } else { fx=from.getX(); tx = to.getX(); }
    	if( from.getY() > to.getY() ) { fy=to.getY(); ty=from.getY(); } else { fy=from.getY(); ty = to.getY(); }
    	if( from.getZ() > to.getZ() ) { fz=to.getZ(); tz=from.getZ(); } else { fz=from.getZ(); tz = to.getZ(); }
    	aabb = AxisAlignedBB.getBoundingBox(fx, fy, fz, tx, ty, tz).addCoord(1.0D,1.0D,1.0D).expand(0.1D,0.1D,0.1D);
    	// Minecraft 1.8
		//aabb = AxisAlignedBB.fromBounds(from.getX(),from.getY(),from.getZ(),to.getX(),to.getY(),to.getZ()).addCoord(1.0D,1.0D,1.0D).expand(0.1D,0.1D,0.1D);
		
		return aabb;
    }
	public static void callChestGen( World world, BlockPos pos, String category, int qty, Logger logger ) {
		Random rnd = new Random();
		// Minecraft 1.7
		TileEntity te = world.getTileEntity(pos.getX(),pos.getY(),pos.getZ());
		// minecraft 1.8
		//TileEntity te = world.getTileEntity(pos);
		
		// Valid category test
		if( ! category.matches("pyramid(Deserty|Jungle)(Chest|Dispenser)|stronghold(Corridor|Library|Crossing)|villageBlacksmith|(bonus|dungeon)Chest|netherFortress")) {
			category = "dungeonChest";
		}
		
		ChestGenHooks info = ChestGenHooks.getInfo(category);
		if( (info != null) && (te instanceof IInventory) && (qty > 0) ) {
			WeightedRandomChestContent.generateChestContents(rnd, info.getItems(rnd), (IInventory)te, qty);
		} else {
			logger.error("Invalid call to ChestGenHooks: Cat="+category+" Qty="+qty);
		}
	}
	public static boolean isValidEntityName(String entity) {
		boolean rv = true;
		// Minecraft 1.7
		// Is there any way to test this?
		// Minecraft 1.8
		//rv = EntityList.isStringValidEntityName(entity);
		return rv;
	}
    public static boolean isAirBlock(World world, BlockPos pos) {
    	// Minecraft .17
    	//return world.isAirBlock(pos)
    	// Minecraft 1.8
   		return world.isAirBlock(pos.getX(),pos.getY(),pos.getZ());
    }
    public static double getDoubleAt(NBTTagList list,int index) {
    	// Minecraft 1.7
    	return list.func_150309_d(index);
    	// minecraft 1.8
    	// return ((NBTTagDouble) list.get(index)).getDouble();
    }
    public static void setId(NBTTagCompound tag, Item item) {
		// Minecraft 1.7
		tag.setShort("id",(short)Item.getIdFromItem(item));
		// Minecraft 1.8
		//tag.setString("id", ""+item.itemRegistry.getNameForObject(item) ); // null string required
    }

}
