package org.steveshipway.cheshirecat.cmd;

import java.util.ArrayList;
import java.util.List;

import org.steveshipway.cheshirecat.CheshireCat;
import org.steveshipway.cheshirecat.item.ItemWand;
import org.steveshipway.cheshirecat.lib.Constants;

import net.minecraft.command.ICommand;
import net.minecraft.command.ICommandSender;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.ChatComponentText;
import net.minecraft.util.MathHelper;
// Minecraft 1.7
import org.steveshipway.blueprints.util17.*;
// Minecraft 1.8
//import net.minecraft.util.BlockPos;
//import org.steveshipway.blueprints.util18.*;

public class BuildCommand implements ICommand {
	  private List aliases;
	  public BuildCommand()
	  {
	    this.aliases = new ArrayList();
	    this.aliases.add("build");
	  }

	  @Override
	  public String getCommandName()
//	  public String getName()
	  {
	    return "build";
	  }

	  @Override
	  public String getCommandUsage(ICommandSender icommandsender)
	  {
	    return "build <blueprint> <x> <y> <z> <r>";
	  }

	  @Override
	  public List getCommandAliases()
//	  public List getAliases()
	  {
	    return this.aliases;
	  }

	  @Override
	  public void processCommand(ICommandSender icommandsender, String[] astring)
//	  public void execute(ICommandSender icommandsender, String[] astring)
	  {
		  int rotation;
		  BlockPos location = null;
		  if(astring.length < 1)
	      {
	    	  icommandsender.addChatMessage(new ChatComponentText("No blueprint name given"));
	    	  return;
	      }
	      // now we can test for a valid blueprint
	      if( ! CheshireCat.portfolio.defines(astring[0]) ) {
	    	  icommandsender.addChatMessage(new ChatComponentText("Invalid blueprint name '" + astring[0] + "'"));
	    	  return;	    	  
	      }
	      if(astring.length < 5)
	      {
	    	  icommandsender.addChatMessage(new ChatComponentText("No build location given"));
	    	  return;
	      }
	      try {
	    	  location = new BlockPos(Integer.parseInt(astring[1]),Integer.parseInt(astring[2]),Integer.parseInt(astring[3]));
	    	  rotation = Integer.parseInt(astring[4]) & 3;
	      } catch( Exception e ) {
	    	  icommandsender.addChatMessage(new ChatComponentText("Invalid coordinates or rotation!"));
	    	  return;
	      }	      
	      //location.posY += player.eyeHeight;
	      if( CheshireCat.portfolio.build(icommandsender.getEntityWorld(), astring[0], location, rotation) ) {
	    	  icommandsender.addChatMessage(new ChatComponentText("Build successful"));
	      } else {
	    	  icommandsender.addChatMessage(new ChatComponentText("Build failed!"));
	      }
	  }

	  @Override
	  public boolean canCommandSenderUseCommand(ICommandSender icommandsender)
//	  public boolean canCommandSenderUse(ICommandSender icommandsender)
	  {
		  // is the player in create mode or holding the wand?
		  EntityPlayer player;
          
          if(icommandsender instanceof EntityPlayer){
                  player = (EntityPlayer)icommandsender;
          } else {
                  return true;
          }
          if( player.capabilities.isCreativeMode ) {
        	  return true;
          }
          ItemStack item = player.getCurrentEquippedItem();
          if (item != null && item.getUnlocalizedName().endsWith(Constants.MODID + "_" + ItemWand.name) ) {
        	  NBTTagCompound nbt = item.getTagCompound();
        	  if( nbt != null && nbt.getBoolean("locked") ) {
		    	  player.addChatMessage(new ChatComponentText("Wand is Locked. Cannot use this command."));
		    	  return false;		    	  
		      }
	    	  return true;
          }
          
    	  player.addChatMessage(new ChatComponentText("You must be in Creative mode or holding a wand"));
		  return false;
	  }

	  @Override
	  public List addTabCompletionOptions(ICommandSender icommandsender,String[] astring)
//	  public List addTabCompletionOptions(ICommandSender icommandsender,String[] astring, BlockPos bp)
	  {
		  // should add the list of blueprints to them...
		  return CheshireCat.portfolio.list(astring[0]);
	  }

	  @Override
	  public boolean isUsernameIndex(String[] astring, int i)
	  {
		  // none of the parameters are usernames
   	      return false;
	  }

	  @Override
	  public int compareTo(Object o)
	  {
	    return 0;
	  }
}
