package org.steveshipway.blueprints.directive;

import java.io.File;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.logging.log4j.Logger;

import net.minecraft.block.Block;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.JsonToNBT;
import net.minecraft.nbt.NBTBase;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagDouble;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.nbt.NBTUtil;
import net.minecraft.server.MinecraftServer;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.world.World;
import net.minecraft.world.chunk.Chunk;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityList;
import net.minecraft.entity.player.EntityPlayer;


// Minecraft 1.7
//import org.steveshipway.blueprints.util17.*;
// Minecraft 1.8
import org.steveshipway.blueprints.util18.*;

import net.minecraft.block.state.IBlockState;
import net.minecraft.util.BlockPos;

public class FrameDirective extends Directive {
	// this is the main one!
	private BlockPos offset;
	private int dv = 0;
	private short itemRot = 0;
	private boolean hasError = false;
	private String itemname = "minecraft:stick";
	private short itemDv = 0;
	
	public FrameDirective(int dv,BlockPos offset, String itemname, int itemDv, int itemRot ) {
		this.offset = offset;
		this.dv = dv & 15;
		this.itemname = itemname;
		this.itemDv = (short) (itemDv & 15);
		this.itemRot = (short) (itemRot & 3);
	}
	public FrameDirective(String definition, Logger logger) {
    	Pattern p = Pattern.compile("(?i)^(?:\\s*frame)?\\s+(-?\\d+)[\\s,]+(-?\\d+)[\\s,]+(-?\\d+)\\s+(\\d)\\s+([a-zA-Z0-9:_-]+)(?:\\((\\d+)\\))?(?:\\s+(\\d+))?\\s*$");
    	Matcher m = p.matcher(definition);
    	if( m.find() ) {
    		int dv = 0;
			// coordinates
    		try {
    			this.offset = new  BlockPos(Integer.parseInt( m.group(1) ),Integer.parseInt( m.group(2) ),Integer.parseInt( m.group(3) ));
    		} catch( Exception e ) {
    			this.hasError = true;
        		logger.error("Cannot parse FRAME definition: "+definition);
        		return;
    		}

			if( m.group(4) != null ) {
    			dv = Integer.parseInt( m.group(4) );
    		} else { dv = 0; }
    		this.dv = dv & 15;

    		if( m.group(5).contains(":") ) {
    			this.itemname = m.group(5);
    		} else {
    			this.itemname = "minecraft:"+m.group(5);
    		}
    		
			if( m.group(6) != null ) {
    			this.itemDv = (short) (Integer.parseInt( m.group(6) ) & 15);
    		} else { this.itemDv = 0; }
    		
			if( m.group(7) != null ) {
    			this.itemRot = (short) (Integer.parseInt( m.group(7) ) & 4);
    		} else { this.itemRot = 0; }
		
    	} else {
    		this.hasError = true;
    		logger.error("Cannot parse FRAME definition: "+definition);
    	}
	}
	
	@Override
	public boolean build(World world, BlockPos location, int r, Logger logger, HashSet softlist) {
		boolean rv;
		BlockPos blockPos = new BlockPos(location);
		NBTTagCompound nbtTag, nbtItem;
		Entity entity;
		int tileX, tileY, tileZ;
		byte facing;
		Item b;
		
		if( world.isRemote ) {
    		logger.error("Block placement called with remote world?  Should never happen.");
			return false;
		}
		if( this.hasError ) {
			logger.error("Frame definition has an error, cannot spawn");
			return false;
		}

		// get block position, rotated
		blockPos = rotateAndAdd(this.offset,r,location);
		
		// Identify the item
		b = Item.getByNameOrId(this.itemname);
		if( b == null ) {
			logger.error("Invalid object in frame ("+this.itemname+"): using a stick");
			b = Item.getByNameOrId("minecraft:stick");
		} else {
			logger.info("Object "+this.itemname+" identified for frame contents: "+b.itemRegistry.getNameForObject(b));
		}

		nbtItem = new NBTTagCompound();
		nbtItem.setByte("Count",(byte) 1);
		nbtItem.setShort("Damage",this.itemDv);

		// Minecraft 1.7
		//nbtItem.setShort("id",(short)Item.getIdFromItem(b));
		// Minecraft 1.8
		nbtItem.setString("id", ""+b.itemRegistry.getNameForObject(b) ); // null string required

		// Now, create the frame entity.
		nbtTag = new NBTTagCompound();
		nbtTag.setString("id","ItemFrame");
		// set map to be frame content
		nbtTag.setTag("Item",nbtItem);
		// set frame options
		nbtTag.setByte("ItemRotation",(byte) this.itemRot);
		nbtTag.setFloat("ItemDropChance",1.0F);
		// rotation in world
		facing = (byte)((this.dv + r)&3);
		
		// Minecraft 1.8
		nbtTag.setByte("Facing",facing);
		tileX = blockPos.getX(); // the block it is IN
		tileY = blockPos.getY();
		tileZ = blockPos.getZ();
		// Minecraft 1.7  // the block it is ON
		//nbtTag.setByte("Direction",facing);
		//tileY = blockPos.getY();
		//switch( facing ) {
		//	case 3: // e
		//		tileX = blockPos.getX()-1;
		//		tileZ = blockPos.getZ();				
		//		break;
		//	case 2: // n ( towards us )
		//		tileX = blockPos.getX();
		//		tileZ = blockPos.getZ()+1;
		//		break;
		//	case 1: // w
		//		tileX = blockPos.getX()+1;
		//		tileZ = blockPos.getZ();
		//		break;
		//	default:  // s
		//		tileX = blockPos.getX();
		//		tileZ = blockPos.getZ()-1;
		//		break;
		//}

		nbtTag.setInteger("TileX", tileX);
		nbtTag.setInteger("TileY", tileY);
		nbtTag.setInteger("TileZ", tileZ);
		
		// set the position tags
		NBTTagList posList = new NBTTagList();
		posList.appendTag(new NBTTagDouble((double)blockPos.getX()+0.5D));
		posList.appendTag(new NBTTagDouble((double)blockPos.getY()+0.5D));
		posList.appendTag(new NBTTagDouble((double)blockPos.getZ()+0.5D));
		nbtTag.setTag("Pos",posList);
		
		// create the entity
		entity = EntityList.createEntityFromNBT(nbtTag, world);
		if( entity == null ) {
			logger.error("Unable to create Frame Entity");
		} else { 
			entity.setPosition((double) blockPos.getX()+0.5D,(double) blockPos.getY(),(double) blockPos.getZ()+0.5D);		
			world.spawnEntityInWorld(entity);
		}
		
		return true;
	}
	
	@Override
	public String asText() {
		String s = "FRAME ";
		if( this.hasError ) {
			s = "# FRAME ";
		}
		s += " " + offset.getX() + "," + offset.getY() + "," + offset.getZ() + "  " + this.dv + " " + this.itemname + "(" + this.itemDv + ") " + this.itemRot;
		return s;
	}
}
