package org.steveshipway.blueprints.util18;

// All Minecraft 1.8 utility functions should be in here!

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.Random;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.logging.log4j.Logger;
import org.steveshipway.cheshirecat.CheshireCat;

import net.minecraft.block.Block;
import net.minecraft.init.Blocks;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.Item;
import net.minecraft.nbt.CompressedStreamTools;
import net.minecraft.nbt.JsonToNBT;
import net.minecraft.nbt.NBTBase;
import net.minecraft.nbt.NBTException;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagDouble;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.WeightedRandomChestContent;
import net.minecraft.world.World;
import net.minecraft.world.biome.BiomeGenBase;
import net.minecraftforge.common.ChestGenHooks;
import net.minecraftforge.common.util.Constants;

// Minecraft 1.7
//import org.steveshipway.blueprints.compat.*;
//import cpw.mods.fml.common.registry.GameRegistry;
// Minecraft 1.8
import net.minecraft.block.BlockLever.EnumOrientation;
import net.minecraft.block.BlockLog.EnumAxis;
import net.minecraft.block.BlockRailBase.EnumRailDirection;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.properties.PropertyDirection;
import net.minecraft.block.properties.PropertyEnum;
import net.minecraft.block.properties.PropertyInteger;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityList;
import net.minecraft.util.EnumFacing.Axis;
import net.minecraft.util.BlockPos;

public class Util {
	// Minecraft 1.8
    public static final PropertyEnum ORIENTATION = PropertyEnum.create("facing", EnumOrientation.class);
    public static final PropertyDirection FACING = PropertyDirection.create("facing");
//    public static final PropertyDirection AXIS = PropertyDirection.create("axis");
    public static final PropertyEnum AXIS = PropertyEnum.create("axis",EnumAxis.class);
    public static final PropertyInteger ROTATION = PropertyInteger.create("rotation",0,15);
    public static final PropertyBool NORTH = PropertyBool.create("north");
    public static final PropertyBool SOUTH = PropertyBool.create("south");
    public static final PropertyBool EAST  = PropertyBool.create("east");
    public static final PropertyBool WEST  = PropertyBool.create("west");
    public static final PropertyEnum SHAPE = PropertyEnum.create("shape",EnumRailDirection.class);
    
	public static Pattern nbtfile = Pattern.compile("\\s*(\\S+\\.nbt)\\s*");
	
	public Util() {
	}
	
    public static IBlockState rotateState( IBlockState blockstate, int r ) {
    	EnumFacing facing;

    	EnumRailDirection raildirection;
    	EnumAxis axis;
    	Integer rotation;
    	IBlockState newstate = blockstate;		
    	Map props = blockstate.getProperties();
    	// rotation for most blocks
    	if( props.containsKey(FACING) ) {      
    		// seems to fail for levers?
    		facing = (EnumFacing) blockstate.getValue(FACING);
            //logger.info("Rotating a " + blockstate.getBlock().toString() + " facing."+facing.toString()+" + "+r);
    		if( facing == EnumFacing.UP || facing == EnumFacing.DOWN ) {
    			// up and down cannot be rotated
    			return blockstate;
    		}
    		if( r == 1 ) {
        		facing = facing.rotateY();
    		} else if( r == 2 ) {
    			facing = facing.rotateY();
    			facing = facing.rotateY();
    		} else if( r == 3 ) {
    			facing = facing.rotateYCCW();
    		}
    		newstate = blockstate.withProperty(FACING, facing);    	
    		return newstate;
    	}
    	if( props.containsKey(ORIENTATION) ) {      
    			EnumOrientation leverfacing;
    			int ir = r;
    			HashMap<EnumOrientation,EnumOrientation> levermap = new HashMap<EnumOrientation,EnumOrientation>();
        		levermap.put(EnumOrientation.NORTH,EnumOrientation.EAST);
        		levermap.put(EnumOrientation.EAST,EnumOrientation.SOUTH);
        		levermap.put(EnumOrientation.SOUTH,EnumOrientation.WEST);
        		levermap.put(EnumOrientation.WEST,EnumOrientation.NORTH);
        		levermap.put(EnumOrientation.UP_X,EnumOrientation.UP_Z);
        		levermap.put(EnumOrientation.DOWN_X,EnumOrientation.DOWN_Z);
        		levermap.put(EnumOrientation.UP_Z,EnumOrientation.UP_X);
        		levermap.put(EnumOrientation.DOWN_Z,EnumOrientation.DOWN_X);
        		
        		leverfacing = (EnumOrientation) blockstate.getValue(ORIENTATION);
                //logger.info("Rotating a lever facing."+leverfacing.toString()+" by "+r);
        		while( ir > 0 ) {
        			ir -= 1;
        			leverfacing = levermap.get(leverfacing);
        		}
                //logger.info("Now have facing."+leverfacing.toString());
	    		newstate = blockstate.withProperty(ORIENTATION, leverfacing);
	    		return newstate;
    	}
    	// rotation for logs, hay bales
    	if( props.containsKey(AXIS) ) {      
    		axis = (EnumAxis) blockstate.getValue(AXIS);
//            logger.info("Rotating an axis."+axis.toString());
    		if( r == 1 || r == 3) {
    			if( axis == EnumAxis.X ) {
    				axis = EnumAxis.Z;
    			} else if( axis == EnumAxis.Z ) {
    				axis = EnumAxis.X;
    			}
    		}
    		newstate = blockstate.withProperty(AXIS, axis);
    		return newstate;
    	}
    	// rotation for banners and signs
    	if( props.containsKey(ROTATION) ) {      
    		newstate = blockstate;
    		rotation = (Integer) blockstate.getValue(ROTATION);
    		rotation = rotation + 4; // 4 positions in a quarter turn
    		if( rotation > 15 ) { rotation -= 16; }
    		newstate = blockstate.withProperty(ROTATION, rotation);
    		return newstate;
    	}
    	// rotation for vines
    	if( props.containsKey(NORTH) ) {      
    		int ir = r;
    		Boolean north = (Boolean) blockstate.getValue(NORTH);
    		Boolean south = (Boolean) blockstate.getValue(SOUTH);
    		Boolean east  = (Boolean) blockstate.getValue(EAST);
    		Boolean west  = (Boolean) blockstate.getValue(WEST);
    		Boolean t;
    		while( ir > 0 ) {
    			ir -= 1;
    			t = east; east = north; north = west; west = south; south = t;
    		}
    		newstate = blockstate.withProperty(NORTH, north).withProperty(SOUTH, south).withProperty(EAST, east).withProperty(WEST, west);
    		return newstate;
    	}
    	// rotation for train tracks; very weird
    	if( props.containsKey(SHAPE) ) {      
    		int ir = r;
    		HashMap<EnumRailDirection,EnumRailDirection> railmap = new HashMap<EnumRailDirection,EnumRailDirection>();
    		railmap.put(EnumRailDirection.NORTH_SOUTH,EnumRailDirection.EAST_WEST);
    		railmap.put(EnumRailDirection.EAST_WEST,EnumRailDirection.NORTH_SOUTH);
    		railmap.put(EnumRailDirection.ASCENDING_NORTH,EnumRailDirection.ASCENDING_EAST);
    		railmap.put(EnumRailDirection.ASCENDING_EAST,EnumRailDirection.ASCENDING_SOUTH);
    		railmap.put(EnumRailDirection.ASCENDING_SOUTH,EnumRailDirection.ASCENDING_WEST);
    		railmap.put(EnumRailDirection.ASCENDING_WEST,EnumRailDirection.ASCENDING_NORTH);
    		railmap.put(EnumRailDirection.NORTH_EAST,EnumRailDirection.SOUTH_EAST);
    		railmap.put(EnumRailDirection.SOUTH_EAST,EnumRailDirection.SOUTH_WEST);
    		railmap.put(EnumRailDirection.SOUTH_WEST,EnumRailDirection.NORTH_WEST);
    		railmap.put(EnumRailDirection.NORTH_WEST,EnumRailDirection.NORTH_EAST);

    		raildirection = (EnumRailDirection) blockstate.getValue(SHAPE);
    		while( ir > 0 ) {
    			ir -= 1;
    			raildirection = railmap.get(raildirection);
    		}
    		newstate = blockstate.withProperty(SHAPE, raildirection);
    		return newstate;
    	}
    	// rotation for mushroom blocks; really weird
    	// XXXX to be done if I can work out how
		return blockstate;
    }
    
    // Utility function.  Convert "{nbt}" or "file.nbt" into a parsed NBT tag
    public static NBTTagCompound parseNBT(String s, String bppath, Logger logger) {
    	NBTTagCompound nbtTag = null;
    	Matcher m = nbtfile.matcher(s);
    	File nbtfile;
    	if( m.find() ) {
    		// we have an NBT file
    		if( s.startsWith(File.separator) || bppath == null ) {
    			nbtfile = new File(m.group(1));
    		} else {
    			nbtfile = new File( bppath+File.separator+"nbt"+File.separator+m.group(1) );
    		}
    		if( ! nbtfile.exists() ) {
				logger.error("Unable to read file "+m.group(1)+" : File does not exist in "+bppath+File.separator+"nbt"+File.separator);
    			return null;
    		} 
    		try {
				nbtTag = CompressedStreamTools.readCompressed(new FileInputStream(nbtfile));
			} catch (IOException e) {
				logger.error("Unable to read file "+m.group(1)+" : "+e.toString());
				return null;
			}
    	} else {
			try {
				// seek out the only function in JsonToNBT that takes a string and returns an NBTTag
				// Minecraft 1.7
		    	//nbtTag = (NBTTagCompound) JsonToNBT.func_150315_a(s); // forge 1.7.10
		    	// Minecraft 1.8
				nbtTag = (NBTTagCompound) JsonToNBT.func_180713_a(s); // forge 1.8
			} catch (NBTException e) {
				nbtTag = null;
				if( logger != null ) {
					logger.error(s);
					logger.error("Unable to parse NBT: "+e.getMessage());
				}
				return null;
			} 
    	}
    	return nbtTag;
    }
    
    // Abstract out getBlock() for portability
    public static Block getBlock(World world, BlockPos location) {
    	Block blk;
    	// Minecraft 1.8
    	blk =  world.getBlockState(location).getBlock();
    	// Minecraft 1.7
    	//blk =  world.getBlock(location.getX(),location.getY(),location.getZ());
    	return blk;
    }
    
    public static IBlockState getState(Block b,int dv) {
    	IBlockState bs;
    	
    	// Minecraft 1.7
    	//bs = new IBlockState(b, dv);
    	// Minecraft 1.8
    	try { 
    		bs = b.getBlockState().getBaseState().getBlock().getStateFromMeta(dv); 
    	} 	catch( Exception e ) {
    		bs = b.getBlockState().getBaseState().getBlock().getDefaultState();
    	}
    	
    	return bs;
    }
    public static IBlockState getBlockState(World world, BlockPos blockPos) {
    	IBlockState bs;
    	// Minecraft 1.7
		//Block curBlock = world.getBlock(blockPos.getX(), blockPos.getY(), blockPos.getZ());
		//int curDv = world.getBlockMetadata(blockPos.getX(), blockPos.getY(), blockPos.getZ());
		//bs = new IBlockState(curBlock,curDv);
    	// Minecraft 1.8
		bs = world.getBlockState(blockPos);
		 
		return bs;
    }
    public static boolean setBlockState(World world, BlockPos blockPos, IBlockState bs) {
    	boolean rv;
    	
    	// Minecraft 1.7
    	//rv = world.setBlock(blockPos.getX(),blockPos.getY(),blockPos.getZ(),bs.getBlock(),bs.getDv(),2);
    	// Minecraft 1.8
    	rv = world.setBlockState(blockPos,bs,2);
    	
    	return rv;
    }
    public static int getDvFromState(IBlockState bs) {
    	int dv;
    	
    	// Minecraft 1.7
    	//dv = bs.getDv();
    	// Minecraft 1.8
    	dv = bs.getBlock().getMetaFromState(bs);
    	
    	return dv;
    }
    public static void setTileEntity(World world,BlockPos blockPos,TileEntity te) {
		// Minecraft 1.8
		world.removeTileEntity(blockPos); // should we instead get and merge?
		world.setTileEntity(blockPos,te);
		// Minecraft 1.7
		//world.removeTileEntity(blockPos.getX(),blockPos.getY(),blockPos.getZ()); // should we instead get and merge?
		//world.setTileEntity(blockPos.getX(),blockPos.getY(),blockPos.getZ(),te);

    }
    public static void setTileEntity(World world,BlockPos blockPos,NBTTagCompound nbtTag) {
    	TileEntity te;
    	NBTTagCompound oldTag;
    	NBTTagCompound newTag;

		te = getTileEntity(world,blockPos);

		oldTag = new NBTTagCompound();
		if( te != null ) {
			te.writeToNBT(oldTag); // note this does not remove existing tags
			oldTag.removeTag("x"); // remove the tags we don't need
			oldTag.removeTag("y");
			oldTag.removeTag("z");
			newTag = mergeNbt(oldTag,nbtTag);
			world.removeTileEntity(blockPos); // should we instead get and merge?
		} else {
			newTag = (NBTTagCompound) nbtTag.copy();
		}
		te = TileEntity.createAndLoadEntity(newTag);
    	
		// Minecraft 1.8
		world.setTileEntity(blockPos,te);

    }
    public static TileEntity getTileEntity(World world, BlockPos blockPos) {
    	// Minecraft 1.7
    	//return world.getTileEntity(blockPos.getX(),blockPos.getY(),blockPos.getZ());
    	// Minecraft 1.8
    	return world.getTileEntity(blockPos);
    }
    public static Item getItemByNameOrId(int id) {
    	// Minecraft 1.7
    	//return Item.getItemById(id);
    	// Minecraft 1.8
    	return Item.getByNameOrId(""+id);    	
    }
    public static Item getItemByNameOrId(String s) {
    	// Minecraft 1.7
        //String modId = "minecraft";
        //String name;
        //if (s.indexOf(":") != -1) {
        //    modId = s.substring(0, s.indexOf(":"));
        //}
        //name = s.substring(s.indexOf(":") + 1);
        //return GameRegistry.findItem(modId, name); // Will return null if no item is found.
    	// Minecraft 1.8
    	return Item.getByNameOrId(s);
    }
    
    public static NBTTagCompound mergeNbt(NBTTagCompound oldTag,NBTTagCompound replaceTag) {
    	NBTTagCompound newTag;
    	
		if( oldTag == null ) {
			return ((NBTTagCompound) replaceTag.copy());
		}
		newTag = (NBTTagCompound) oldTag.copy();
		if( replaceTag == null ) {
			return newTag;
		}
		newTag.merge(replaceTag); // do new items replace old ones, or are old preserved?  Do list items get preserved?
    	return newTag;
    }
    // fix the tag (not much to do in Mc 1.8)
    public static NBTTagCompound fixChestNbt(Logger logger, NBTTagCompound nbt) {
    	NBTTagList items;
    	
    	if( !nbt.hasKey("id",Constants.NBT.TAG_STRING) || !(nbt.getString("id").contentEquals("Chest")) ) {
    		logger.warn("Error in chest NBT: Setting id to Chest");
    		nbt.setString("id","Chest");
    	}
    	items = nbt.getTagList("Items",Constants.NBT.TAG_COMPOUND);
    	if( items != null ) {
	    	for (int i = 0; i < items.tagCount(); ++i) {
	    		NBTTagCompound item = (NBTTagCompound) items.getCompoundTagAt(i);
	    		if( !item.hasKey("id") ) {
	    			logger.warn("Correcting NBT: missing id on chest item");
	    			item.setString("id", "minecraft:stick");
	    		}
	    	}
    	}
    	return nbt;
    }
    public static NBTTagCompound fixSpawnerNbt(Logger logger, NBTTagCompound nbt) {
    	NBTTagList potentials;
    	
    	if( !nbt.hasKey("id",Constants.NBT.TAG_STRING) || !(nbt.getString("id").contentEquals("MobSpawner") ) ) {
    		logger.warn("Error in spawner NBT: Setting id to MobSpawner");
    		nbt.setString("id", "MobSpawner");
    	}
    	    	
    	// this can stop crashes under 1.7; 1.8 just produces pigs
    	potentials = nbt.getTagList("SpawnPotentials",Constants.NBT.TAG_COMPOUND);
    	if( potentials != null ) {
	    	for (int i = 0; i < potentials.tagCount(); ++i) {
	    		NBTTagCompound pot = (NBTTagCompound) potentials.getCompoundTagAt(i);
	    		if( !pot.hasKey("Type",Constants.NBT.TAG_STRING) || (pot.getString("Type").length()<1) ) {
	    			logger.warn("No Spawner potential Type set:  Setting Type to Chicken (was: '"+pot.getString("Type")+"')");
	    			pot.setString("Type", "Chicken");
	    		}
	    	}
    	}

		if( ((nbt.getString("EntityId")==null)||(nbt.getString("EntityId").length()<1)) && (potentials.tagCount() > 0) ) {
			try {
				NBTBase firstPotential =  Util.getIndex(potentials, 0);				
				nbt.setString("EntityId", ((NBTTagCompound)firstPotential).getString("Type") );
				nbt.setTag("SpawnData", ((NBTTagCompound)firstPotential).getTag("Properties").copy() );
			} catch( Exception e ) {
				logger.warn("Possible problem in duplicating primary spawn potential to base definition");
				logger.warn("Error: "+e.toString());
			}
		}

    	// this stops crashes under 1.7 where entityid is unset or null
    	// it doesnt validate that the entityId is real, though
    	if( !nbt.hasKey("EntityId",Constants.NBT.TAG_STRING) || (nbt.getString("EntityId").length() < 1) ) {
    		logger.warn("Error in spawner NBT:  No EntityId set: Setting entity to Chicken");
    		nbt.setString("EntityId", "Chicken");
    	}
    	
    	//logger.info("Spawner NBT tag: "+nbt.toString());
    	return nbt;
    }
    
    public static Entity getEntityByID(World world, int entityId)  {
   	 for(Object o: world.getLoadedEntityList())
        {
            if(((Entity)o).getEntityId() == entityId )
            {
                return ((Entity)o);
            }
        }
        return null;
    }
    public static NBTTagCompound getIndex(NBTTagList list,int index) {
		// Minecraft 1.8
		return (NBTTagCompound) list.get(index);
		// Minecraft 1.7
		//return list.getCompoundTagAt(index);

    }
    public static String getBiome(World world, BlockPos location ) {
		// Minecraft 1.8
        return world.getBiomeGenForCoords(location).biomeName;
        // Minecraft 1.7
        //return world.getBiomeGenForCoords(location.getX(),location.getZ()).biomeName;    	
    }
    // convert a metablock type to a block
    public static IBlockState metaBlock(World world, BlockPos pos, String name, int dv) {
    	Random r = new Random();
    	Block b = Blocks.air;
    	IBlockState bs;
        String biome = Util.getBiome(world,pos);
    	if( name.contentEquals( "STONE") ) {
    		// change according to biome
    		// change to sandstone if in desert
    		if( biome.contains("Desert") ) {
        		b = Blocks.sandstone;
    		} else if(biome.contains("Hell") ) {
    			b = Blocks.nether_brick;
    		} else if(biome.contains("Mesa") ) {
    			b = Blocks.red_sandstone;
    		} else {
    			switch( r.nextInt(10) ) {
    			case 0:
    				b = Blocks.monster_egg;
    				dv = 1; // cobblestone; 3=mossy
    				break;
    			case 1:
    			case 2:
    				b = Blocks.stonebrick;
    				break;
    			default:
    				b = Blocks.cobblestone;
    			}
    		}
    	} else if( name.contentEquals( "STONE_SLAB") ) {
        		// change according to biome
        		// change to sandstone if in desert
        		if( biome.contains("Desert") ) {
            		b = Blocks.stone_slab;
        			if(dv>7) { dv = 9; } else { dv = 1; }
        		} else if(biome.contains("Hell") ) {
        			b = Blocks.stone_slab;
        			if(dv>7) { dv = 14; } else { dv = 6; }
        		} else if(biome.contains("Mesa") ) {
        			b = Blocks.stone_slab2;
        			if(dv>7) { dv = 8; } else { dv = 0; }
        		} else {
       				b = Blocks.stone_slab;
        			if(dv>7) { dv = 11; } else { dv = 3; }
      			}
    	} else if( name.contentEquals( "STONE_STEPS") ||  name.contentEquals( "STONE_STAIRS" )) {
    		// change according to biome
    		// preserve original DV
    		if( biome.contains("Desert") ) {
        		b = Blocks.sandstone_stairs;
    		} else if(biome.contains("Hell") ) {
    			b = Blocks.nether_brick_stairs;
    		} else if(biome.contains("Mesa") ) {
    			b = Blocks.red_sandstone_stairs;
    		} else {
        		b = Blocks.stone_stairs;
    		}
    	} else if( name.contentEquals( "OLD_STONE" )) {
    		// change according to biome
    		if( biome.contains("Desert") ) {
           		b = Blocks.sandstone;
    		} else if(biome.contains("Hell") ) {
    			b = Blocks.nether_brick;
    		} else {
        		// can be mossy or have silverfish
    			switch( r.nextInt(10) ) {
	    			case 0:
	    				b = Blocks.monster_egg;
	    				dv = 1; // cobblestone 
	    				break;
	    			case 1:
	    				b = Blocks.monster_egg;
	    				dv = 3; // mossy brick
	    				break;
	    			case 2:
	    			case 3:
	    				b = Blocks.mossy_cobblestone;
	    				break;
	    			case 4:
	    				dv = 0;
	    				b = Blocks.stonebrick;
	    				break;
	    			case 5:
	    				dv = 1; // mossy brick
	    				b = Blocks.stonebrick;
	    				break;
	    			case 6:
	    				dv = 2; // cracked brick
	    				b = Blocks.stonebrick;
	    				break;
	            	default:
	            		b = Blocks.cobblestone;
	            		break;
    			}

    		}
    	} else if( name.contentEquals( "WOOD" ) || name.contentEquals( "PLANKS" ) ) {
    		// change DV according to biome.  order of tests is important
    		b = Blocks.planks;
			if( biome.matches(".*(Ice|Taiga).*") ) { // spruce
				dv = 1;
			} else if( biome.matches("(Roofed|Hell).*") ) { // dark oak
				dv = 5;
			} else if( biome.matches("Jungle.*") ) { // jungle
				dv = 3;
			} else if( biome.matches("Savanna.*") ) { // acacia
				dv = 4;
			} else if( biome.matches(".*(Plains|Forest|Desert|Mesa|Plateau).*") ) { // birch
				dv = 2;
			} else { // oak
				dv = 0;
			}
    	} else if( name.contentEquals( "WOOD_STEPS" ) ||  name.contentEquals( "WOOD_STAIRS" )) {
    		// change according to biome
			if( biome.matches(".*(Ice|Taiga).*") ) { // spruce
				b = Blocks.spruce_stairs;
			} else if( biome.matches("(Roofed|Hell).*") ) { // dark oak
				b = Blocks.dark_oak_stairs;
			} else if( biome.matches("Jungle.*") ) { // jungle
				b = Blocks.jungle_stairs;
			} else if( biome.matches("Savanna.*") ) { // acacia
				b = Blocks.acacia_stairs;
			} else if( biome.matches(".*(Plains|Forest|Desert|Mesa|Plateau).*") ) { // birch
				b = Blocks.birch_stairs;
			} else { // oak
				b = Blocks.oak_stairs;
			}
		} else if( name.contentEquals( "WOOD_SLAB" )) {
    		b = Blocks.wooden_slab;
    		if( dv > 7 ) { dv = 8; } else { dv = 0; } // upper or lower
			if( biome.matches(".*(Ice|Taiga).*") ) { // spruce
				dv += 1;
			} else if( biome.matches("(Roofed|Hell).*") ) { // dark oak
				dv += 5;
			} else if( biome.matches("Jungle.*") ) { // jungle
				dv += 3;
			} else if( biome.matches("Savanna.*") ) { // acacia
				dv += 4;
			} else if( biome.matches(".*(Plains|Forest|Desert|Mesa|Plateau).*") ) { // birch
				dv += 2;
			}  // oak
    	} else if( name.contentEquals("DOOR") ) {
			if( biome.matches(".*(Ice|Taiga).*") ) { // spruce
				b = Blocks.spruce_door;
			} else if( biome.matches("(Roofed|Hell).*") ) { // dark oak
				b = Blocks.dark_oak_door;
			} else if( biome.matches("Jungle.*") ) { // jungle
				b = Blocks.jungle_door;
			} else if( biome.matches("Savanna.*") ) { // acacia
				b = Blocks.acacia_door;
			} else if( biome.matches(".*(Plains|Forest|Desert|Mesa|Plateau).*") ) { // birch
				b = Blocks.birch_door;
			} else { // oak
				b = Blocks.oak_door;
			}
    	} else if( name.contentEquals("FENCE") ) {
			if( biome.matches(".*(Ice|Taiga).*") ) { // spruce
				b = Blocks.spruce_fence;
			} else if( biome.matches("Roofed.*") ) { // dark oak
				b = Blocks.dark_oak_fence;
			} else if( biome.matches("Hell.*") ) { // nether
				b = Blocks.nether_brick_fence;
			} else if( biome.matches("Jungle.*") ) { // jungle
				b = Blocks.jungle_fence;
			} else if( biome.matches("Savanna.*") ) { // acacia
				b = Blocks.acacia_fence;
			} else if( biome.matches(".*(Plains|Forest|Desert|Mesa|Plateau).*") ) { // birch
				b = Blocks.birch_fence;
			} else { // oak
				b = Blocks.oak_fence;
			}
    	} else if( name.contentEquals("GATE") ) {
			if( biome.matches(".*(Ice|Taiga).*") ) { // spruce
				b = Blocks.spruce_fence_gate;
			} else if( biome.matches("(Roofed|Hell).*") ) { // dark oak
				b = Blocks.dark_oak_fence_gate;
			} else if( biome.matches("Jungle.*") ) { // jungle
				b = Blocks.jungle_fence_gate;
			} else if( biome.matches("Savanna.*") ) { // acacia
				b = Blocks.acacia_fence_gate;
			} else if( biome.matches(".*(Plains|Forest|Desert|Mesa|Plateau).*") ) { // birch
				b = Blocks.birch_fence_gate;
			} else { // oak
				b = Blocks.oak_fence_gate;
			}
    	} else if( name.contentEquals("GLASS") ) {  // random coloured glass pane
    		b = Blocks.stained_glass;
    		dv = r.nextInt(16);
    	} else if( name.contentEquals("GLASS_PANE") ) {  // random coloured glass pane
    		b = Blocks.stained_glass_pane;
    		dv = r.nextInt(16);
    	} else {
    		b = Block.getBlockFromName(name);
    		if( b == null ) {
    			b = Blocks.air;
    		}
    	}
    	bs = Util.getState(b,dv);
    	return bs;
    }
	public static AxisAlignedBB getBB(BlockPos from, BlockPos to) {
    	AxisAlignedBB aabb;
    	
    	// Minecraft 1.7
    	//double fx,fy,fz,tx,ty,tz;
    	//if( from.getX() > to.getX() ) { fx=to.getX(); tx=from.getX(); } else { fx=from.getX(); tx = to.getX(); }
    	//if( from.getY() > to.getY() ) { fy=to.getY(); ty=from.getY(); } else { fy=from.getY(); ty = to.getY(); }
    	//if( from.getZ() > to.getZ() ) { fz=to.getZ(); tz=from.getZ(); } else { fz=from.getZ(); tz = to.getZ(); }
    	//aabb = AxisAlignedBB.getBoundingBox(fx, fy, fz, tx+1.0D, ty+1.0D, tz+1.0D);
    	// Minecraft 1.8
		aabb = AxisAlignedBB.fromBounds(from.getX(),from.getY(),from.getZ(),to.getX(),to.getY(),to.getZ()).addCoord(1.0D,1.0D,1.0D).expand(0.1D,0.1D,0.1D);
		
		return aabb;
    }
	public static void callChestGen( World world, BlockPos pos, String category, int qty, Logger logger ) {
		Random rnd = new Random();
		// Minecraft 1.7
		//TileEntity te = world.getTileEntity(pos.getX(),pos.getY(),pos.getZ());
		// minecraft 1.8
		TileEntity te = world.getTileEntity(pos);
		
		// Valid category test
		if( ! category.matches("pyramid(Deserty|Jungle)(Chest|Dispenser)|stronghold(Corridor|Library|Crossing)|villageBlacksmith|(bonus|dungeon)Chest|netherFortress")) {
			category = "dungeonChest";
		}
		
		ChestGenHooks info = ChestGenHooks.getInfo(category);
		if( (info != null) && (te instanceof IInventory) && (qty > 0) ) {
			WeightedRandomChestContent.generateChestContents(rnd, info.getItems(rnd), (IInventory)te, qty);
		} else {
			logger.error("Invalid call to ChestGenHooks: Cat="+category+" Qty="+qty);
		}
	}
	public static boolean isValidEntityName(String entity) {
		boolean rv = true;
		// Minecraft 1.7
		// Is there any way to test this?
		// Minecraft 1.8
		rv = EntityList.isStringValidEntityName(entity);
		return rv;
	}
	public static boolean isAirBlock(World world, BlockPos pos) {
    	// Minecraft 1.8
    	return world.isAirBlock(pos);
    	// Minecraft 1.7
   		//return world.isAirBlock(pos.getX(),pos.getY(),pos.getZ());
    }
	    public static double getDoubleAt(NBTTagList list,int index) {
    	// Minecraft 1.7
    	//return list.func_150309_d(index);
    	// minecraft 1.8
    	return ((NBTTagDouble) list.get(index)).getDouble();
    }
    public static void setId(NBTTagCompound tag, Item item) {
		// Minecraft 1.7
		//tag.setShort("id",(short)Item.getIdFromItem(item));
		// Minecraft 1.8
		tag.setString("id", ""+item.itemRegistry.getNameForObject(item) ); // null string required
    }
}
